#!/bin/sh

# snort2pfsense.sh (snort to pfSense shell script)

# Author: Josep Pujadas i Jubany
# Date: 22-june-2007 
# Purpose: Send to a pfSense box the hosts to block (for @pf_timer seconds) from an snort sensor (external to the pfSense box)  

# Please, read http://www.bellera.cat/josep/snort2pfsense for any snort2pfsense information & license terms 

# Where snort2pfsense files are ...
s2p="/root/snort2pfsense"
s2p_log="/var/log/snort"

# Where is the pfSense box ...
pf_ip="192.168.XXX.1"
pf_pass="********"
pf_sleep="0"

# Where snort MySQL database is ... 
host="127.0.0.1"
dbname="snort_database"
user="admin_snort_database"
passwd="********"

# Who to send BLOCK/UNBLOCK mails ...
# mailto=""
mailto="user@domain.example"

touch $s2p_log/snort2pfsense.log
chmod 400 $s2p_log/snort2pfsense.log

case "$1" in
	"")	
		echo
		echo "RUNNING snort2pfsense ..."
		echo
		while true; do
			/usr/local/bin/mysql -h $host -u $user -p$passwd $dbname < $s2p/snort2pfsense_block.sql | /usr/bin/grep -v "INET_NTOA" > $s2p/snort2pfsense_blocking.txt
			/usr/local/bin/mysql -h $host -u $user -p$passwd $dbname < $s2p/snort2pfsense_unblock.sql | /usr/bin/grep -v "INET_NTOA" > $s2p/snort2pfsense_unblocking.txt
			blocking="`/usr/bin/wc -l $s2p/snort2pfsense_blocking.txt | /usr/bin/awk -F' ' '{ print $1 }'`"
			unblocking="`/usr/bin/wc -l $s2p/snort2pfsense_unblocking.txt | /usr/bin/awk -F' ' '{ print $1 }'`"
			connect="no"	
			if [ "$blocking" != "0" ]; then
				connect="yes"
			else
				if [ "$unblocking" != "0" ]; then
					connect="yes"
				fi
			fi
			if [ "$connect" = "yes" ]; then
				echo "#!/usr/local/bin/expect" > $s2p/snort2pfsense.exp
				chmod 700 $s2p/snort2pfsense.exp
				echo "spawn /usr/bin/ssh admin@$pf_ip" >> $s2p/snort2pfsense.exp
				echo "expect assword {send $pf_pass\r}" >> $s2p/snort2pfsense.exp
				echo "expect option {send 8\r}" >> $s2p/snort2pfsense.exp
			fi
			if [ "$unblocking" != "0" ]; then
				cat $s2p/snort2pfsense_unblocking.txt | while read ip_address; do
                                	string="UNBLOCKING $ip_address (`/usr/bin/dig -x $ip_address +short | /usr/bin/head -n 1`) at `date +"%d/%m/%Y %H:%M:%S"`"
					echo $string
					# pfSense deletes IPs after an expiration time in its snort2c pf table:
					# cat /cf/conf/config.xml | grep snort2c
					# <command>/usr/bin/nice -n20 /usr/local/sbin/expiretable -t 1800 snort2c</command>			
					# We delete the IP from here just to be sure ...   
					echo 'send "pfctl -t snort2c -T del '$ip_address'\r"' >> $s2p/snort2pfsense.exp
					if [ "$mailto" != "" ] ; then
						echo "" | mail -s "$string" $mailto
					fi
				done
			fi
                        if [ "$blocking" != "0" ]; then
                                cat $s2p/snort2pfsense_blocking.txt | while read ip_address; do
                                        string="BLOCKING $ip_address (`/usr/bin/dig -x $ip_address +short | /usr/bin/head -n 1`) at `date +"%d/%m/%Y %H:%M:%S"`"
                                        echo $string
                                        echo 'send "pfctl -t snort2c -T add '$ip_address'\r"' >> $s2p/snort2pfsense.exp
                                        echo 'send "pfctl -k '$ip_address'\r"' >> $s2p/snort2pfsense.exp
                                        if [ "$mailto" != "" ] ; then
                                                echo "" | mail -s "$string" $mailto
                                        fi
                                done
                        fi
			if [ "$connect" = "yes" ]; then
				echo 'send "exit\r"' >> $s2p/snort2pfsense.exp
				echo "expect option {send 0\r}" >> $s2p/snort2pfsense.exp
				echo "expect eof" >> $s2p/snort2pfsense.exp
				echo "----------------------------------------------------------------------------------" >> $s2p_log/snort2pfsense.log
				echo `date +"%d/%m/%Y %H:%M:%S"` >> $s2p_log/snort2pfsense.log	
				$s2p/snort2pfsense.exp >> $s2p_log/snort2pfsense.log
				sleep $pf_sleep
			fi
			sleep 1 
		done
		;;
	"whitelist_view")
	 	echo
 		echo "Here is your snort2pfsense whitelist ..."
		echo 	
		/usr/local/bin/mysql -h $host -u $user -p$passwd $dbname < $s2p/snort2pfsense_whitelist_view.sql | /usr/bin/grep -v "ip_" | while read ip_mask; do
			ip=`echo $ip_mask | awk '{print $1}'`
			mask=`echo $ip_mask | awk '{print $2}'`
			if [ "$ip" = "127.0.0.1" ]; then
				echo "$ip (localhost)"
			else
				if [ "$mask" = "255.255.255.255" ]; then	
					echo "$ip (`/usr/bin/dig -x $ip +short | /usr/bin/head -n 1`)"
				else
					echo "$ip/$mask (WARNING! This is an entire network, not a host)"
				fi
			fi
		done
		echo
		echo "snort2pfsense will never block these addresses & networks!"
		echo 
		;;
        "create")
                echo
                echo "Creating snort2pfsense tables into your $dbname snort database ..."
                echo
                /usr/local/bin/mysql -h $host -u $user -p$passwd $dbname < $s2p/snort2pfsense_create.sql
		echo
		echo "Finished!"
		echo	
		;;
	"whitelist")
                echo
                echo "Updating snort2pfsense whitelist from snort2pfsense_whitelist.sql ..."
                echo
                /usr/local/bin/mysql -h $host -u $user -p$passwd $dbname < $s2p/snort2pfsense_whitelist.sql
                echo
                echo "Finished!"
		$s2p/snort2pfsense.sh whitelist_view
		echo
                ;;
        "blocked")
                echo
                echo "Hosts blocked by snort2pfsense at `date +"%d/%m/%Y %H:%M:%S"` ..."
                echo
                /usr/local/bin/mysql -h $host -u $user -p$passwd $dbname < $s2p/snort2pfsense_blocked.sql  | /usr/bin/grep -v "INET_NTOA" | while read adreca_ip; do
                                echo "$adreca_ip (`/usr/bin/dig -x $adreca_ip +short | /usr/bin/head -n 1`)"
		done
                echo
                echo "Each host is unblocked @pf_timer seconds after blocking it!"
	 	echo "Type 'pfctl -rt snort2c -vT show' at pfSense console. You should see the same result !!!"
		echo
                ;;
	"flush")
		$s2p/snort2pfsense.sh blocked
		echo "Flushing snort2pfsense_pftabled & and snort2c PF pfSense tables ..."
		echo
		/usr/local/bin/mysql -h $host -u $user -p$passwd $dbname < $s2p/snort2pfsense_flush.sql
                echo "#!/usr/local/bin/expect" > $s2p/snort2pfsense.exp
                chmod 700 $s2p/snort2pfsense.exp
                echo "spawn /usr/bin/ssh admin@$pf_ip" >> $s2p/snort2pfsense.exp
                echo "expect assword {send $pf_pass\r}" >> $s2p/snort2pfsense.exp
                echo "expect option {send 8\r}" >> $s2p/snort2pfsense.exp
                echo 'send "pfctl -t snort2c -T flush\r"' >> $s2p/snort2pfsense.exp
                echo 'send "exit\r"' >> $s2p/snort2pfsense.exp
                echo "expect option {send 0\r}" >> $s2p/snort2pfsense.exp
                echo "expect eof" >> $s2p/snort2pfsense.exp
		echo "----------------------------------------------------------------------------------" >> $s2p_log/snort2pfsense.log
		echo `date +"%d/%m/%Y %H:%M:%S"` >> $s2p_log/snort2pfsense.log
		$s2p/snort2pfsense.exp >> $s2p_log/snort2pfsense.log
		sleep $pf_sleep
		echo "Done!" 				
		;;
	"since")
		/usr/local/bin/since $s2p_log/snort2pfsense.log | more
		;;
	"alert")
		/usr/local/bin/since /var/log/snort/alert | more
		;;
	"start")
                pid=`ps -axw | /usr/bin/grep "snort2pfsense.sh" | /usr/bin/grep "??" | /usr/bin/awk -F' ' '{ print $1 }'`
		if [ "$pid" = "" ] ; then
			/usr/sbin/daemon $s2p/snort2pfsense.sh >/dev/null
			echo "snort2pfsense.sh started as daemonized!"
		else	
			echo "Found snortpfsense.sh running at pid $pid. I can't start it a new time!" 
		fi
		;;
	"stop")
		ps -axw | /usr/bin/grep "snort2pfsense.sh" | /usr/bin/grep "??" | /usr/bin/awk -F' ' '{ print $1 }' | while read pid; do   
			kill $pid
			echo "snort2pfsense.sh pid $pid killed!"
		done	
		;;
	"restart")
		$s2p/snort2pfsense.sh stop
		$s2p/snort2pfsense.sh start
		;;
esac

